#if !defined ( _SLOG )
#include "slog.h"
#endif  /*  if !defined ( _SLOG )  */


/*  LOW Level API  */

void SLOG_InitGlobalData( void );

SLOG_STREAM *SLOG_OpenStream( const char *path, const char *mode );

int SLOG_CloseStream( SLOG_STREAM *slog );

int SLOG_STM_CreateBbufs( SLOG_STREAM  *slog );

void SLOG_STM_FreeBbufs( SLOG_STREAM  *slog );

int SLOG_WriteInitFrameDir( SLOG_STREAM *slog );

int SLOG_ReadFrameDir( SLOG_STREAM *slog );

int SLOG_WriteFrameDirEntryGiven(       FILE             *slog_fd,
                                  const SLOG_dir_entry_t *framedir_entry );

int SLOG_ReadFrameDirEntry( FILE             *slog_fd,
                            SLOG_dir_entry_t *framedir_entry );

int SLOG_WriteUpdatedFrameDirEntry( SLOG_STREAM     *slog );



/*  LOWER Level API  */

int SLOG_STM_IdxOfCurFrameDirEntry( const SLOG_STREAM *slog );

void SLOG_STM_Init( SLOG_STREAM *slog );

void SLOG_STM_UpdateFrameDirHdrGiven(       SLOG_STREAM       *slog,
                                      const SLOG_fptr          prevdir,
                                      const SLOG_fptr          nextdir,
                                      const SLOG_uint32        Nframe );

int SLOG_STM_IncrementFrameDirHdr( SLOG_STREAM       *slog );

int SLOG_STM_WriteFrameDirHdr( SLOG_STREAM       *slog );

int SLOG_STM_UpdateFrameDirEntry( SLOG_STREAM         *slog );

void SLOG_STM_UpdateFrameHdrGiven(       SLOG_STREAM  *slog,
                                   const SLOG_uint32   Nbytes_inc_i,
                                   const SLOG_uint32   Nrec_inc_i,
                                   const SLOG_uint32   Nbytes_cur_i,
                                   const SLOG_uint32   Nrec_cur_i,
                                   const SLOG_uint32   Nbytes_out_i,
                                   const SLOG_uint32   Nrec_out_i );

void SLOG_STM_UpdateFrameHdr( SLOG_STREAM  *slog );

int SLOG_STM_DepositFrameHdr( const SLOG_framehdr_t *frame_hdr,
                                    filebuf_t       *fbuf );

int SLOG_STM_WithdrawFrameHdr( SLOG_framehdr_t *frame_hdr,
                               filebuf_t       *fbuf );

int SLOG_STM_UpdateBBuf_endtime(       SLOG_intvlrec_blist_t *out_bbuf,
                                 const SLOG_intvlrec_blist_t *in_bbuf,
                                 const SLOG_dir_entry_t       frame_dir_entry );

int SLOG_STM_DepositBBuf( const SLOG_intvlrec_blist_t *Bbuf,
                                filebuf_t             *fbuf );

int SLOG_STM_WithdrawBBuf(       SLOG_intvlrec_blist_t *Bbuf,
                           const SLOG_recdefs_table_t  *slog_recdefs,
                                 filebuf_t             *fbuf );

int SLOG_STM_UpdateFRAME( SLOG_STREAM *slog );

int SLOG_STM_WriteFRAME( SLOG_STREAM *slog );

int SLOG_STM_ReadFRAME( SLOG_STREAM *slog );

int SLOG_STM_UpdateWriteFRAMEDIR( SLOG_STREAM *slog );

void SLOG_STM_InitAgainBeforeAddRec( SLOG_STREAM *slog );

void SLOG_STM_PrintRecDefs( const SLOG_STREAM *slog, FILE *outfd );

void SLOG_STM_PrintFrameDirHdr( const SLOG_STREAM *slog, FILE *outfd );

void SLOG_STM_PrintFrameDirEntries( const SLOG_STREAM *slog, FILE *outfd );



/*  LOWEST Level API  */

int SLOG_Bbuf_SetAtHead( SLOG_intvlrec_blist_t *slog_bbuf );

int SLOG_Bbuf_PointAtNULL( const SLOG_intvlrec_blist_t *slog_bbuf );

const SLOG_intvlrec_t *
SLOG_Bbuf_GetIntvlRec( SLOG_intvlrec_blist_t *slog_bbuf );

void SLOG_Bbuf_Init( SLOG_intvlrec_blist_t *slog_bbuf );

int SLOG_Bbuf_AddIntvlRec(       SLOG_intvlrec_blist_t *slog_bbuf,
                           const SLOG_intvlrec_t       *irec );

void SLOG_Bbuf_DelNode( SLOG_intvlrec_blist_t *slog_bbuf );

void SLOG_Bbuf_DelAllNodes( SLOG_intvlrec_blist_t *slog_bbuf );

void SLOG_Bbuf_CopyBBufVal( const SLOG_intvlrec_blist_t *src,
                                  SLOG_intvlrec_blist_t *dest );



/*  Component level supporting subroutines  */

	/* SLOG_RecDef_xxxx methods */

int SLOG_RecDef_IsVarIntvl( const SLOG_rectype_t  rectype );

SLOG_N_assocs_t
SLOG_RecDef_NumOfAssocs( const SLOG_recdefs_table_t   *recdefs,
                         const SLOG_intvltype_t        intvltype,
                         const SLOG_bebit_t            bebit_0,
                         const SLOG_bebit_t            bebit_1 );

SLOG_N_args_t 
SLOG_RecDef_NumOfArgs( const SLOG_recdefs_table_t   *recdefs,
                       const SLOG_intvltype_t        intvltype,
                       const SLOG_bebit_t            bebit_0,
                       const SLOG_bebit_t            bebit_1 );

	/* SLOG_Varg_xxxx methods */

SLOG_vtrarg_t *SLOG_Varg_Create( const SLOG_N_args_t  N_args,
                                 const SLOG_arg_t    *args );

void SLOG_Varg_Free( SLOG_vtrarg_t *vtr_arg );

SLOG_vtrarg_t *SLOG_Varg_Copy( const SLOG_vtrarg_t *src );

int SLOG_Varg_Print( const SLOG_vtrarg_t *vtr_arg,  FILE *outfd );

	/* SLOG_bebit(s)_xxxx methods */

int SLOG_bebit_valid( const SLOG_bebit_t bebit );

SLOG_bebit_t SLOG_bebit_conv( const SLOG_bebit_t bebit );

int SLOG_bebits_valid( const SLOG_bebits_t bebits );

SLOG_bebits_t SLOG_bebits_encode( const SLOG_bebit_t bebit_0,
                                  const SLOG_bebit_t bebit_1 );

int SLOG_bebits_decode( const SLOG_bebits_t bebits, SLOG_bebit_t *bebit_ary );
